package de.ozgcloud.nachrichten.postfach.osiv2.transfer;

import java.util.Collection;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Stream;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Service;

import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.osiv2.OsiPostfachException;
import de.ozgcloud.nachrichten.postfach.osiv2.config.Osi2PostfachProperties;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.api.MessageExchangeApi;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.MessageExchangeReceiveMessage;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.MessageExchangeReceiveMessagesResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service
@ConditionalOnProperty(prefix = Osi2PostfachProperties.PREFIX, name = "enabled", havingValue = "true")
@RequiredArgsConstructor
public class PostfachApiFacadeService {

	private final MessageExchangeApi messageExchangeApi;
	private final Osi2RequestMapper osi2RequestMapper;
	private final Osi2ResponseMapper osi2ResponseMapper;

	static final int MAX_NUMBER_RECEIVED_MESSAGES = 100;

	public void sendMessage(PostfachNachricht nachricht) {
		messageExchangeApi.sendMessage(
				osi2RequestMapper.mapMailboxId(nachricht),
				osi2RequestMapper.mapOutSendMessageRequestV2(nachricht)
		);
	}

	public Stream<PostfachNachricht> receiveMessages() {
		return Optional.ofNullable(receiveMessagesResponse().getMessages())
				.stream()
				.flatMap(Collection::stream)
				.map(this::fetchMessageByGuid);
	}
	
	private MessageExchangeReceiveMessagesResponse receiveMessagesResponse() {
		return Optional.ofNullable(messageExchangeApi.receiveMessages(MAX_NUMBER_RECEIVED_MESSAGES, 0))
				.orElseThrow(() -> new OsiPostfachException("Expect non empty response!", null));
	}

	PostfachNachricht fetchMessageByGuid(final MessageExchangeReceiveMessage message) {
		var messageReply = messageExchangeApi.getMessage(message.getGuid());
		return osi2ResponseMapper.toPostfachNachricht(messageReply);
	}

	public void deleteMessage(final String messageId) {
		messageExchangeApi.deleteMessage(UUID.fromString(messageId));
	}
}
