package de.ozgcloud.nachrichten.postfach.osiv2.extension;

import static com.github.tomakehurst.wiremock.client.WireMock.*;
import static de.ozgcloud.nachrichten.postfach.osiv2.factory.JwtFactory.*;

import org.junit.jupiter.api.extension.AfterAllCallback;
import org.junit.jupiter.api.extension.AfterEachCallback;
import org.junit.jupiter.api.extension.BeforeAllCallback;
import org.junit.jupiter.api.extension.BeforeEachCallback;
import org.junit.jupiter.api.extension.ExtensionContext;

import com.github.tomakehurst.wiremock.WireMockServer;

import de.ozgcloud.nachrichten.postfach.osiv2.factory.JwtFactory;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Getter
@RequiredArgsConstructor
public class OsiMockServerExtension implements BeforeAllCallback, AfterAllCallback, BeforeEachCallback, AfterEachCallback {

	private WireMockServer serviceKontoMockServer;
	private WireMockServer postfachFacadeMockServer;

	@Override
	public void beforeAll(ExtensionContext context) {
		setupPostfachFacadeMock();
		setupServiceKontoMock();
	}

	@Override
	public void afterEach(ExtensionContext context) {
		postfachFacadeMockServer.resetAll();
		serviceKontoMockServer.resetAll();
	}

	@Override
	public void afterAll(ExtensionContext context) {
		serviceKontoMockServer.shutdown();
		postfachFacadeMockServer.shutdown();
	}

	private void setupPostfachFacadeMock() {
		postfachFacadeMockServer = new WireMockServer(0);
		postfachFacadeMockServer.start();
	}

	private void setupServiceKontoMock() {
		serviceKontoMockServer = new WireMockServer(0);
		serviceKontoMockServer.start();
	}

	public String getAccessTokenUrl() {
		return serviceKontoMockServer.baseUrl() + "/access-token";
	}

	public String getPostfachFacadeUrl() {
		return postfachFacadeMockServer.baseUrl();
	}

	@Override
	public void beforeEach(ExtensionContext context) {
		serviceKontoMockServer.stubFor(post("/access-token")
				.withHeader("Content-Type", equalTo("application/x-www-form-urlencoded"))
				.withFormParam("grant_type", equalTo("client_credentials"))
				.withFormParam("client_id", equalTo(CLIENT_ID))
				.withFormParam("scope", equalTo(String.join(" ", CLIENT_SCOPES)))
				.withFormParam("resource", equalTo(RESOURCE_URN))
				.willReturn(okJson(JwtFactory.createTokenResponse(
						JwtFactory.createAccessTokenExampleWithExpireIn(900)
				))));
	}
}
